@@ -5,60 +5,52 @@ use 5.008_004;
 use strict;
 use warnings FATAL => 'all';
 
-our $VERSION = '0.0400001';
+our $VERSION = '0.040001';
 
 use Graph;
 
 sub toposort_graph {
-  my $self = shift;
-  my (%opts) = @_;
+    my $self = shift;
+    my ($schema, %opts) = @_;
 
-  my $g = Graph->new;
+    my $g = Graph->new;
 
-  my @source_names = $self->sources;
+    my @source_names = $schema->sources;
 
-  my %table_source = map { 
-    $self->source($_)->name => $_
-  } @source_names;
+    my %table_source = map { 
+        $schema->source($_)->name => $_
+    } @source_names;
 
-  foreach my $name ( @source_names ) {
-    my $source = $self->source($name);
-    $g->add_vertex($name);
+    foreach my $name ( @source_names ) {
+        my $source = $schema->source($name);
+        $g->add_vertex($name);
 
-    foreach my $rel_name ( $source->relationships ) {
-      next if grep { $_ eq $rel_name } @{$opts{skip}{$name}};
-      my $rel_info = $source->relationship_info($rel_name);
+        foreach my $rel_name ( $source->relationships ) {
+            next if grep { $_ eq $rel_name } @{$opts{skip}{$name}};
+            my $rel_info = $source->relationship_info($rel_name);
 
-      if ( $rel_info->{attrs}{is_foreign_key_constraint} ) {
-        $g->add_edge(
-          $table_source{$self->source($rel_info->{source})->name},
-          $name,
-        );
-      }
+            if ( $rel_info->{attrs}{is_foreign_key_constraint} ) {
+                $g->add_edge(
+                    $table_source{$schema->source($rel_info->{source})->name},
+                    $name,
+                );
+            }
+        }
     }
-  }
 
-  return $g;
+    return $g;
 }
 
 sub toposort {
-  my $self = shift;
-
-  my ($g, @rv);
-  eval {
-    $g = $self->toposort_graph(@_);
-    @rv = $g->toposort();
-  }; if ($@) {
-    if ($g) {
-      my @c = $g->find_a_cycle;
-      if (@c) {
-        warn "Cycle found: '" . join("' -> '", @c) . "'\n";
-      }
+    my $self = shift;
+    my $schema;
+    if (ref($self) && $self->isa('DBIx::Class::Schema')) {
+        $schema = $self;
     }
-    die $@;
-  }
-
-  return @rv;
+    else {
+        $schema = shift(@_);
+    }
+    return $self->toposort_graph($schema, @_)->toposort();
 }
 
 1;
@@ -87,6 +79,10 @@ If you have a cycle in your relationships
       },
   );
 
+Alternately:
+
+  my @toposorted_sourcenames = DBIx::Class::TopoSort->toposort($schema);
+
 =head1 DESCRIPTION
 
 This adds a method to L<DBIx::Class::Schema> which returns the full list of
@@ -117,6 +113,15 @@ section on TOPOLOGICAL SORT.
 This method will throw an error if there are any cycles in your tables. You will
 need to specify the skip parameter (described below) to break those cycles.
 
+=head2 toposort (Class method)
+
+Alternately, if you do not wish to use TopoSort as a component, you can call it
+as a class method on this class. The toposort() method is smart enough to
+distinguish.
+
+Note: toposort_graph() does B<not> distinguish - it assumes it will be called
+with the C<$schema> object passed in.
+
 =head2 toposort_graph
 
 This returns a L<Graph> object with a vertex for every source and an edge for
